@@ -1,6 +1,27 @@
-/* $Id: cfg.c,v 1.22 2002/03/14 06:42:21 prahl Exp $
+/* cfg.c - Read config files and provide lookup routines
 
-     purpose : Read config files and provide lookup routines
+Copyright (C) 1995-2002 The Free Software Foundation
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 2
+of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+This file is available from http://sourceforge.net/projects/latex2rtf/
+
+Authors:
+    1995-1997 Ralf Schlatterbeck
+    1998-2000 Georg Lehner
+    2001-2002 Scott Prahl
 
  * LEG200698 I would have prefered to make the reading of the language file
  * separate, since the language is known some steps after reading the
@@ -34,20 +55,16 @@ static ConfigInfoT configinfo[] =
 	{"direct.cfg", NULL, 0, FALSE},
 	{"fonts.cfg", NULL, 0, FALSE},
 	{"ignore.cfg", NULL, 0, FALSE},
+	{"style.cfg", NULL, 0, FALSE},
 	{"english.cfg", NULL, 0, FALSE},
-
 };
+
 #define CONFIG_SIZE (sizeof(configinfo) / sizeof(ConfigInfoT))
 #define BUFFER_INCREMENT 1024
 
-extern void     Fatal(const char *fmt,...);
-void            ParseError(const char *fmt,...);
-char           *ReadUptoMatch(FILE * infile, const char *scanchars);
+char *ReadUptoMatch(FILE * infile, const char *scanchars);
 
-static char * g_cfg_filename;
-
-static int 
-cfg_compare(ConfigEntryT ** el1, ConfigEntryT ** el2)
+static int cfg_compare(ConfigEntryT ** el1, ConfigEntryT ** el2)
 /****************************************************************************
  * purpose:  compare-function for bsearch
  * params:   el1, el2: Config Entries to be compared
@@ -56,17 +73,22 @@ cfg_compare(ConfigEntryT ** el1, ConfigEntryT ** el2)
 	return strcmp((*el1)->TexCommand, (*el2)->TexCommand);
 }
 
-FILE *
+static FILE *
 try_path(const char *path, const char *file)
+/****************************************************************************
+ * purpose:  append path to .cfg file name and open
+             return NULL upon failure,
+             return filepointer otherwise
+ ****************************************************************************/
 {
 	char * both;
 	FILE * fp=NULL;
-	int    lastchar;
-		
-	diagnostics(4, "trying path=<%s> file=<%s>", path, file);
-	
+	size_t lastchar;
+			
 	if (path==NULL || file == NULL) return NULL;
 
+	diagnostics(4, "trying path=<%s> file=<%s>", path, file);
+
 	lastchar = strlen(path);
 
 	both = malloc(strlen(path) + strlen(file) + 2);
@@ -87,8 +109,8 @@ try_path(const char *path, const char *file)
 	return fp;
 }
 
-FILE *
-open_cfg(const char *name)
+void *
+open_cfg(const char *name, int quit_on_error)
 /****************************************************************************
 purpose: open config by trying multiple paths
  ****************************************************************************/
@@ -97,6 +119,8 @@ purpose: open config by trying multiple paths
 	char		   *lib_path;
 	FILE           *fp;
 
+	diagnostics(3,"Open file in cfg directories <%s>",name);
+
 /* try path specified on the line */
 	fp=try_path(g_config_path, name);
 	if (fp) return fp;
@@ -118,8 +142,8 @@ purpose: open config by trying multiple paths
 		free(env_path);
 	}		
 
-/* last resort.  try LIBDIR */
-	lib_path = strdup(LIBDIR);
+/* last resort.  try CFGDIR */
+	lib_path = strdup(CFGDIR);
 	if (lib_path) {
 		p = lib_path;
 		while (p) {
@@ -135,94 +159,104 @@ purpose: open config by trying multiple paths
 	}
 
 /* failed ... give some feedback */
+	if (quit_on_error) {
 	diagnostics(WARNING, "Cannot open the latex2rtf .cfg files");
 	diagnostics(WARNING, "Locate the directory containing the .cfg files and");
 	diagnostics(WARNING, "   (1) define the environment variable RTFPATH, *or*");
 	diagnostics(WARNING, "   (2) use command line path option \"-P /path/to/cfg/file\", *or*");
-	diagnostics(WARNING, "   (3) recompile latex2rtf with LIBDIR defined properly");
+	diagnostics(WARNING, "   (3) recompile latex2rtf with CFGDIR defined properly");
 	diagnostics(WARNING, "Current RTFPATH: %s", getenv("RTFPATH"));
-	diagnostics(WARNING, "Current  LIBDIR: %s", LIBDIR);
-	diagnostics(ERROR,   " Giving up.  Have a nice day.");
+	diagnostics(WARNING, "Current  CFGDIR: %s", CFGDIR);
+	diagnostics(ERROR,   " Giving up.  Please don't hate me.");
+	}
 	return NULL;
 }
 
-
-/***/
 static size_t 
-read_cfg(FILE * cfgfile
-	 ,ConfigEntryT *** pointer_array
-	 ,bool do_remove_backslash
-)
+read_cfg(FILE * cfgfile, ConfigEntryT *** pointer_array, bool do_remove_backslash)
 /****************************************************************************
  * purpose: Read config file and provide sorted lookup table
  ****************************************************************************/
-/* @modifies pointer_array@ */
 {
 	size_t          bufindex = 0, bufsize = 0;
 	char           *line, *cmdend;
 
 	if (*pointer_array == NULL) {
-		if ((*pointer_array = malloc(BUFFER_INCREMENT * sizeof(char *))) == NULL) {
-			Fatal("Cannot allocate memory for pointer list\n");
-		}
+		*pointer_array = malloc(BUFFER_INCREMENT * sizeof(char *));
 		bufsize = BUFFER_INCREMENT;
+		if (*pointer_array == NULL)
+			diagnostics(ERROR,"Cannot allocate memory for pointer list");
 	}
+	
 	while ((line = ReadUptoMatch(cfgfile, "\n")) != NULL) {
-		(void) getc(cfgfile);	/* skip newline */
+		
+		/* skip newline */
+		getc(cfgfile);	
+		
 		/* Skip leading white space */
-		while (isspace((unsigned char) *line)) {
+		while (isspace((unsigned char) *line)) 
 			line++;
-		}
-		if (*line == '#' || *line == '\0') {
+		
+		/* Skip comment line */
+		if (*line == '#') 
 			continue;
-		}
+		
+		/* Skip blank line */
+		if (*line == '\0') 
+			continue;
+			
+		/* Find period that terminates command */
 		cmdend = strrchr(line, '.');
-		if (cmdend == NULL) {
-			ParseError("Bad config file, missing final period\nBad line is \"%s\"", line);
-		}
+		if (cmdend == NULL)
+			diagnostics(ERROR,"Bad config file, missing final period\nBad line is \"%s\"", line);
+		
+		/* Replace period with NULL */
 		*cmdend = '\0';
+		
+		/* Skip backslash if specified */
 		if (do_remove_backslash) {
-			if (*line != '\\') {
-				ParseError("Bad config file, missing initial'\\'\nBad line is\"%s\"", line);
-			} else {
+			if (*line != '\\')
+				diagnostics(ERROR,"Bad config file, missing initial'\\'\nBad line is\"%s\"", line);
+			else
 				line++;
-			}
 		}
+		
+		/* resize buffer if needed */
 		if (bufindex >= bufsize) {
-			/*
-			 * LEG210698*** Here we know, that pointer array is
-			 * not null! What to do with the second allocation?
-			 */
-			if ((*pointer_array
-			     = malloc((bufsize += BUFFER_INCREMENT) * sizeof(char *))
-			     ) == NULL
-				) {
-				Fatal("Cannot allocate memory for pointer list\n");
-			}
+			bufsize += BUFFER_INCREMENT;
+			*pointer_array = realloc(*pointer_array, bufsize * sizeof(char *));
+			if (*pointer_array == NULL) 
+				diagnostics(ERROR,"Cannot allocate memory for pointer list");
 		}
-		line = strdup(line);
+		
+		/* find start of definition */
+		line=strdup(line);
 		cmdend = strchr(line, ',');
-		if (cmdend == NULL) {
-			ParseError("Bad config file, missing ',' between elements\nBad line is\"%s\"", line);
-		}
-		*cmdend++ = '\0';
+		if (cmdend == NULL) 
+			diagnostics(ERROR,"Bad config file, missing ',' between elements\nBad line is\"%s\"", line);
+		
+		/* terminate command */
+		*cmdend = '\0';
 
-		if (((*pointer_array)[bufindex] = malloc(sizeof(ConfigEntryT))) == NULL) {
-			Fatal("Cannot allocate memory for config entry\n");
-		}
+		(*pointer_array)[bufindex] = malloc(sizeof(ConfigEntryT));
+		
+		if ((*pointer_array)[bufindex] == NULL)
+			diagnostics(ERROR,"Cannot allocate memory for config entry");
+		
 		(*pointer_array)[bufindex]->TexCommand = line;
-		(*pointer_array)[bufindex]->RtfCommand = cmdend;
+		(*pointer_array)[bufindex]->RtfCommand = cmdend+1;
 		bufindex++;
 	}
+		
 	qsort(*pointer_array
 	      ,bufindex
 	      ,sizeof(**pointer_array)
 	      ,(fptr) cfg_compare
 		);
+
 	return bufindex;
 }
 
-
 void 
 ReadCfg(void)
 /****************************************************************************
@@ -232,10 +266,13 @@ ReadCfg(void)
 {
 	size_t          i;
 	FILE           *fp;
-
+	char		   *fname;
+	
 	for (i = 0; i < CONFIG_SIZE; i++) {
-		g_cfg_filename = configinfo[i].filename;
-		fp = open_cfg(configinfo[i].filename);
+		fname = configinfo[i].filename;
+		fp = (FILE *) open_cfg(fname, TRUE);
+		diagnostics(4,"reading config file %s", fname);
+		
 		configinfo[i].config_info_size
 			= read_cfg(fp
 				   ,&(configinfo[i].config_info)
@@ -271,7 +308,7 @@ search_rtf(const char *theTexCommand, int WhichCfg)
 		);
 }
 
-size_t 
+int 
 SearchRtfIndex(const char *theTexCommand, int WhichCfg)
 /****************************************************************************
  * purpose:  search theTexCommand in a specified config data and return
@@ -354,7 +391,7 @@ ReadLanguage(char *lang)
 	strcpy(langfn, lang);
 	strcat(langfn, ".cfg");
 
-	fp = open_cfg(langfn);
+	fp = (FILE *) open_cfg(langfn, TRUE);
 	free(langfn);
 
 	configinfo[LANGUAGE_A].config_info_size
@@ -379,34 +416,7 @@ ConvertBabelName(char *name)
 		ConvertString(s);
 }
 
-/* @exits@ */
-void 
-ParseError(const char *fmt,...)
-{
-	va_list         ap;
-
-	fprintf(stderr, "%s: %s: ", progname, g_cfg_filename);
-	va_start(ap, fmt);
-	(void) vfprintf(stderr, fmt, ap);
-	va_end(ap);
-	fprintf(stderr, "\n");
-	exit(EXIT_FAILURE);
-}
-
-void 
-Fatal(const char *fmt,...)
-{
-	va_list         ap;
-
-	fprintf(stderr, "%s: Fatal error: ", progname);
-	va_start(ap, fmt);
-	(void) vfprintf(stderr, fmt, ap);
-	va_end(ap);
-	fprintf(stderr, "\n");
-	exit(EXIT_FAILURE);
-}
-
-static char *buffer;
+static char		*buffer = NULL;
 static size_t   bufsize = 0;
 
 #define CR (char) 0x0d
@@ -416,21 +426,22 @@ static size_t   bufsize = 0;
  * This function assumes there are no '\0' characters in the input.
  * if there are any, they are ignored.
  */
-char           *
-ReadUptoMatch(FILE * infile, /* @observer@ */ const char *scanchars)
+char *
+ReadUptoMatch(FILE * infile, const char *scanchars)
 {
 	size_t          bufindex = 0;
 	int             c;
 
-	if (feof(infile) != 0) {
+	if (feof(infile) != 0)
 		return NULL;
-	}
+	
 	if (buffer == NULL) {
-		if ((buffer = malloc(BUFFER_INCREMENT)) == NULL) {
-			Fatal("Cannot allocate memory for input buffer\n");
-		}
+		buffer = malloc(BUFFER_INCREMENT * sizeof(char));
+		if (buffer == NULL) 
+			diagnostics(ERROR, "Cannot allocate memory for input buffer");
 		bufsize = BUFFER_INCREMENT;
 	}
+	
 	while ((c = getc(infile)) != EOF ) {
 	
 		if (c == CR || c == LF)
@@ -439,25 +450,21 @@ ReadUptoMatch(FILE * infile, /* @observer@ */ const char *scanchars)
 		if (strchr(scanchars, c))
 			break;
 			
-		if (c == (int) '\0') {
+		if (c == (int) '\0') 
 			continue;
-		}
-/*		if (c == (int) '\n') {
-			linenumber++;
-		}
-*/		buffer[bufindex++] = (char) c;
+
+		buffer[bufindex++] = (char) c;
 		if (bufindex >= bufsize) {
-			if ((buffer = realloc(buffer, bufsize += BUFFER_INCREMENT)) == NULL) {
-				Fatal("Cannot allocate memory for input buffer\n");
-			}
+			bufsize += BUFFER_INCREMENT;
+			buffer = realloc(buffer, bufsize);
+			if (buffer == NULL)
+				diagnostics(ERROR,"Cannot allocate memory for input buffer");
 		}
 	}
 	buffer[bufindex] = '\0';
-	if (c != EOF) {
-		ungetc(c, infile);	/* LEG210698*** lclint, GNU libc
-					 * doesn't say what's the return
-					 * value */
-	}
+	if (c != EOF)
+		ungetc(c, infile);
+
 	return buffer;
 }
 
