@@ -1,6 +1,27 @@
-/*  $Id: commands.c,v 1.51 2002/04/04 03:11:24 prahl Exp $
+/* commands.c - Defines subroutines to translate LaTeX commands to RTF
+
+Copyright (C) 1995-2002 The Free Software Foundation
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 2
+of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+This file is available from http://sourceforge.net/projects/latex2rtf/
  
-    Defines subroutines to translate LaTeX commands to RTF
+Authors:
+    1995-1997 Ralf Schlatterbeck
+    1998-2000 Georg Lehner
+    2001-2002 Scott Prahl
 */
 
 #include <stdlib.h>
@@ -24,13 +45,16 @@
 #include "graphics.h"
 
 typedef struct commandtag {
-	char           *cpCommand;			/* LaTeX command name without \ */
-	void (*func) (int);	        		/* function converting LaTex-cmd to Rtf-cmd */
-	int             param;	    		/* used in various ways */
+	char	*cpCommand;			/* LaTeX command name without \ */
+	void	(*func) (int);	    /* function to convert LaTeX to RTF */
+	int		param;	    		/* used in various ways */
 } CommandArray;
 
 static int      iEnvCount = 0;			/* number of active environments */
 static CommandArray *Environments[100];	/* list of active environments */
+static int g_par_indent_array[100];
+static int g_left_indent_array[100];
+static int g_right_indent_array[100];
 
 static CommandArray commands[] = {
 	{"begin", CmdBeginEnd, CMD_BEGIN},
@@ -107,6 +131,7 @@ static CommandArray commands[] = {
 	{"normalfont",   CmdTextNormal, F_TEXT_NORMAL_2},
 	{"mathnormal",   CmdTextNormal, F_TEXT_NORMAL_3},
 
+	{"raggedright", CmdAlign, PAR_RAGGEDRIGHT},
 	{"centerline", CmdAlign, PAR_CENTERLINE},
 	{"vcenter",    CmdAlign, PAR_VCENTER},
 	
@@ -118,6 +143,7 @@ static CommandArray commands[] = {
 	{"BibTeX", CmdLogo, CMD_BIBTEX},
 	{"AmSTeX", CmdLogo, CMD_AMSTEX},
 	{"AmSLaTeX", CmdLogo, CMD_AMSLATEX},
+	{"LyX", CmdLogo, CMD_LYX},
 	
 	/* ---------- SPECIAL CHARACTERS ------------------- */
 	{"hat", CmdHatChar, 0},
@@ -167,9 +193,11 @@ static CommandArray commands[] = {
 	{"cleardoublepage", CmdNewPage, NewPage},
 	{"newpage", CmdNewPage, NewColumn},
 	{"pagebreak", CmdNewPage, NewPage},
-	{"mbox", CmdBox, 0},
-	{"hbox", CmdBox, 0},
-	{"vbox", CmdBox, 0},
+	{"mbox",   CmdBox, BOX_MBOX},
+	{"hbox",   CmdBox, BOX_HBOX},
+	{"vbox",   CmdBox, BOX_VBOX},
+	{"fbox",   CmdBox, BOX_FBOX},
+	{"parbox", CmdBox, BOX_PARBOX},
 	{"frenchspacing", CmdIgnore, 0},
 	{"nonfrenchspacing", CmdIgnore, 0},
 	{"include", CmdIgnoreParameter, No_Opt_One_NormParam},	/* should not happen*/
@@ -180,6 +208,10 @@ static CommandArray commands[] = {
 	{"onecolumn", CmdColumn, One_Column},
 	{"twocolumn", CmdColumn, Two_Column},
 	{"includegraphics", CmdGraphics, 0},
+	{"epsffile", CmdGraphics, 1},
+	{"epsfbox", CmdGraphics, 2},
+	{"BoxedEPSF", CmdGraphics, 3},
+	{"psfig", CmdGraphics, 4},
 	{"includegraphics*", CmdGraphics, 0},
 	{"moveleft", CmdLength, 0},
 	{"moveright", CmdLength, 0},
@@ -189,16 +221,18 @@ static CommandArray commands[] = {
 	{"label", CmdLabel, LABEL_LABEL},
 	{"ref", CmdLabel, LABEL_REF},
 	{"pageref", CmdLabel, LABEL_PAGEREF},
-	{"cite", CmdLabel, LABEL_CITE},
+	{"cite", CmdCite, CITE_CITE},
 	{"bibliography", CmdBibliography, 0},
+	{"bibliographystyle", CmdBibliographyStyle, 0},
 	{"bibitem", CmdBibitem, 0},
 	{"newblock", CmdNewblock, 0},
 	{"newsavebox", CmdIgnoreParameter, No_Opt_One_NormParam},
 	{"usebox", CmdIgnoreParameter, No_Opt_One_NormParam},
-	{"fbox", CmdIgnoreParameter, No_Opt_One_NormParam},
+/*	{"fbox", CmdIgnoreParameter, No_Opt_One_NormParam}, */
 	{"quad", CmdQuad, 1},
 	{"qquad", CmdQuad, 2},
-	{"textsuperscript", CmdSuperscript, 0},
+	{"textsuperscript", CmdSuperscript, 1},
+	{"textsubscript", CmdSubscript, 1},
 	{"hspace", CmdIgnoreParameter, No_Opt_One_NormParam},
 	{"hspace*", CmdIgnoreParameter, No_Opt_One_NormParam},
 	{"vspace", CmdVspace, 0},
@@ -237,7 +271,6 @@ static CommandArray commands[] = {
 	{"framebox", CmdIgnoreParameter, Two_Opt_One_NormParam},
 	{"sbox", CmdIgnoreParameter, No_Opt_Two_NormParam},
 	{"savebox", CmdIgnoreParameter, Two_Opt_Two_NormParam},
-	{"parbox", CmdIgnoreParameter, One_Opt_Two_NormParam},
 	{"rule", CmdIgnoreParameter, One_Opt_Two_NormParam},
 	{"raisebox", CmdIgnoreParameter, Two_Opt_Two_NormParam},
 	{"newfont", CmdIgnoreParameter, No_Opt_Two_NormParam},
@@ -249,18 +282,21 @@ static CommandArray commands[] = {
 	{"typein", CmdIgnoreParameter, One_Opt_One_NormParam},
 	{"marginpar", CmdIgnoreParameter, One_Opt_One_NormParam},
 	{"baselineskip", Cmd_OptParam_Without_braces, 0},
+	{"psfrag", CmdIgnoreParameter, No_Opt_Two_NormParam},
 	{"lineskip", Cmd_OptParam_Without_braces, 0},
 	{"vsize", Cmd_OptParam_Without_braces, 0},
 	{"setbox", Cmd_OptParam_Without_braces, 0},
 	{"thanks", CmdFootNote, FOOTNOTE_THANKS},
 	{"bibliographystyle", CmdIgnoreParameter, No_Opt_One_NormParam},
 	{"let", CmdIgnoreLet, 0},
-	{"cline", CmdIgnoreParameter, No_Opt_One_NormParam},
 	{"multicolumn", CmdMultiCol, 0},
 	{"frac", CmdFraction, 0},
 	{"dfrac", CmdFraction, 0},
     {"Frac", CmdFraction, 0},
 	{"sqrt", CmdRoot, 0},
+    {"lim",  CmdLim, 0},
+    {"limsup",  CmdLim, 1},
+    {"liminf",  CmdLim, 2},
     {"int",  CmdIntegral, 0},
     {"iint",  CmdIntegral, 3},
     {"iiint",  CmdIntegral, 4},
@@ -269,11 +305,24 @@ static CommandArray commands[] = {
     {"left", CmdLeftRight, 0},
     {"right", CmdLeftRight, 1},
     {"stackrel", CmdStackrel, 0},
+    {"matrix", CmdMatrix, 0},
+	{"leftrightarrows", CmdArrows, LEFT_RIGHT},
+	{"leftleftarrows", CmdArrows, LEFT_LEFT},
+	{"rightrightarrows", CmdArrows, RIGHT_RIGHT},
+	{"rightleftarrows", CmdArrows, RIGHT_LEFT},
+	{"longleftrightarrows", CmdArrows, LONG_LEFTRIGHT},
+	{"longrightleftarrows", CmdArrows, LONG_RIGHTLEFT},
 	{"nonumber",CmdNonumber, EQN_NO_NUMBER},
 	{"char",CmdChar,0},
 	{"htmladdnormallink",CmdHtml,LABEL_HTMLADDNORMALREF},
 	{"htmlref",CmdHtml, LABEL_HTMLREF},
 	{"nobreakspace", CmdNonBreakSpace, 0},
+	{"abstract", CmdAbstract, 1},
+	{"endinput", CmdEndInput, 0},
+	{"textcolor", CmdTextColor, 0},
+	{"citename", CmdCiteName, 0},
+	{"shortcite", CmdCite, CITE_SHORT},
+
 	{"", NULL, 0}
 };
 
@@ -306,6 +355,9 @@ static CommandArray PreambleCommands[] = {
 	{"hyphenation", CmdHyphenation, 0},
 	{"def", CmdNewDef, DEF_DEF},
 	{"newcommand", CmdNewDef, DEF_NEW},
+	{"providecommand", CmdNewDef, DEF_NEW},
+	{"DeclareRobustCommand", CmdNewDef, DEF_NEW},
+	{"DeclareRobustCommand*", CmdNewDef, DEF_NEW},
 	{"renewcommand", CmdNewDef, DEF_RENEW},
 	{"newenvironment", CmdNewEnvironment, DEF_NEW},
 	{"renewenvironment", CmdNewEnvironment, DEF_RENEW},
@@ -340,6 +392,10 @@ static CommandArray PreambleCommands[] = {
 	{"htmladdnormallink",CmdHtml, LABEL_HTMLADDNORMALREF},
 	{"htmlref",CmdHtml, LABEL_HTMLREF},
 	{"nobreakspace", CmdNonBreakSpace, 0},
+	{"signature", CmdSignature, 0},
+	{"hline", CmdHline, 0},
+	{"cline", CmdHline, 1},
+	{"ifx", CmdIf, 0},
 	{"", NULL, 0}
 };				/* end of list */
 
@@ -364,11 +420,6 @@ static CommandArray FigureCommands[] = {
 	{"", NULL, 0}
 };
 
-static CommandArray TabbingCommands[] = {
-	{"kill", CmdTabkill, 0},/* a line that ends with a \kill command produces no output */
-	{"", NULL, 0}
-};
-
 static CommandArray LetterCommands[] = {
 	{"opening", CmdOpening, 0},
 	{"closing", CmdClosing, 0},
@@ -389,6 +440,11 @@ static CommandArray GermanModeCommands[] = {
 	{"", NULL, 0}
 };
 
+static CommandArray CzechModeCommands[] = {
+	{"uv", CmdCzechAbbrev, 0},
+	{"", NULL, 0}
+};
+
 static CommandArray FrenchModeCommands[] = {
 /*    {"degree", CmdFrenchAbbrev, DEGREE}, */
     {"ier", CmdFrenchAbbrev, IERF},
@@ -407,10 +463,95 @@ static CommandArray FrenchModeCommands[] = {
     {"secundo", CmdFrenchAbbrev, SECUNDO},
     {"tertio", CmdFrenchAbbrev, TERTIO},
     {"quarto", CmdFrenchAbbrev, QUARTO},
+    {"inferieura", CmdFrenchAbbrev, INFERIEURA},
+    {"superieura", CmdFrenchAbbrev, SUPERIEURA},
+    {"lq", CmdFrenchAbbrev, FRENCH_LQ},
+    {"rq", CmdFrenchAbbrev, FRENCH_RQ},
+    {"lqq", CmdFrenchAbbrev, FRENCH_LQQ},
+    {"rqq", CmdFrenchAbbrev, FRENCH_RQQ},
+    {"pointvirgule", CmdFrenchAbbrev, POINT_VIRGULE},
+    {"pointexclamation", CmdFrenchAbbrev, POINT_EXCLAMATION},
+    {"pointinterrogation", CmdFrenchAbbrev, POINT_INTERROGATION},
+    {"dittomark", CmdFrenchAbbrev, DITTO_MARK},
+    {"deuxpoints", CmdFrenchAbbrev, DEUX_POINTS},
     {"fup", CmdFrenchAbbrev, FUP},
+    {"up", CmdFrenchAbbrev, FUP},
+    {"LCS", CmdFrenchAbbrev, LCS},
+    {"FCS", CmdFrenchAbbrev, FCS},
 	{"", NULL, 0}
 };
 
+/********************************************************************/
+/* commands for Russian Mode */
+/********************************************************************/
+static CommandArray RussianModeCommands[] = {
+	{ "CYRA" , CmdCyrillicChar , 0xC0 },
+	{ "CYRB" , CmdCyrillicChar , 0xC1},
+	{ "CYRV" , CmdCyrillicChar , 0xC2},
+	{ "CYRG" , CmdCyrillicChar , 0xC3},
+	{ "CYRD" , CmdCyrillicChar , 0xC4},
+	{ "CYRE" , CmdCyrillicChar , 0xC5},
+	{ "CYRZH" , CmdCyrillicChar , 0xC6},
+	{ "CYRZ" , CmdCyrillicChar , 0xC7},
+	{ "CYRI" , CmdCyrillicChar , 0xC8},
+	{ "CYRISHRT" , CmdCyrillicChar , 0xC9},
+	{ "CYRK" , CmdCyrillicChar , 0xCA},
+	{ "CYRL" , CmdCyrillicChar , 0xCB},
+	{ "CYRM" , CmdCyrillicChar , 0xCC},
+	{ "CYRN" , CmdCyrillicChar , 0xCD},
+	{ "CYRO" , CmdCyrillicChar , 0xCE},
+	{ "CYRP" , CmdCyrillicChar , 0xCF},
+	{ "CYRR" , CmdCyrillicChar , 0xD0},	
+	{ "CYRS" , CmdCyrillicChar , 0xD1},
+	{ "CYRT" , CmdCyrillicChar , 0xD2},
+	{ "CYRU" , CmdCyrillicChar , 0xD3},
+	{ "CYRF" , CmdCyrillicChar , 0xD4},
+	{ "CYRH" , CmdCyrillicChar , 0xD5},
+	{ "CYRC" , CmdCyrillicChar , 0xD6},
+	{ "CYRCH" , CmdCyrillicChar , 0xD7},
+	{ "CYRSH" , CmdCyrillicChar , 0xD8},
+	{ "CYRCHSH" , CmdCyrillicChar , 0xD9},
+	{ "CYRHRDSN" , CmdCyrillicChar , 0xDA},
+	{ "CYRERY" , CmdCyrillicChar , 0xDB},
+	{ "CYRSFTSN" , CmdCyrillicChar , 0xDC},
+	{ "CYREREV" , CmdCyrillicChar , 0xDD},
+	{ "CYRYU" , CmdCyrillicChar , 0xDE},
+	{ "CYRYA" , CmdCyrillicChar , 0xDF},
+	{ "cyra" , CmdCyrillicChar , 0xE0 },
+	{ "cyrb" , CmdCyrillicChar , 0xE1},
+	{ "cyrv" , CmdCyrillicChar , 0xE2},
+	{ "cyrg" , CmdCyrillicChar , 0xE3},
+	{ "cyrd" , CmdCyrillicChar , 0xE4},
+	{ "cyre" , CmdCyrillicChar , 0xE5},
+	{ "cyrzh" , CmdCyrillicChar , 0xE6},
+	{ "cyrz" , CmdCyrillicChar , 0xE7},
+	{ "cyri" , CmdCyrillicChar , 0xE8},
+	{ "cyrishrt" , CmdCyrillicChar , 0xE9},
+	{ "cyrk" , CmdCyrillicChar , 0xEA},
+	{ "cyrl" , CmdCyrillicChar , 0xEB},
+	{ "cyrm" , CmdCyrillicChar , 0xEC},
+	{ "cyrn" , CmdCyrillicChar , 0xED},
+	{ "cyro" , CmdCyrillicChar , 0xEE},
+	{ "cyrp" , CmdCyrillicChar , 0xEF},
+	{ "cyrr" , CmdCyrillicChar , 0xF0},	
+	{ "cyrs" , CmdCyrillicChar , 0xF1},
+	{ "cyrt" , CmdCyrillicChar , 0xF2},
+	{ "cyru" , CmdCyrillicChar , 0xF3},
+	{ "cyrf" , CmdCyrillicChar , 0xF4},
+	{ "cyrh" , CmdCyrillicChar , 0xF5},
+	{ "cyrc" , CmdCyrillicChar , 0xF6},
+	{ "cyrch" , CmdCyrillicChar , 0xF7},
+	{ "cyrsh" , CmdCyrillicChar , 0xF8},
+	{ "cyrchsh" , CmdCyrillicChar , 0xF9},
+	{ "cyrhrdsn" , CmdCyrillicChar , 0xFA},
+	{ "cyrery" , CmdCyrillicChar , 0xFB},
+	{ "cyrsftsn" , CmdCyrillicChar , 0xFC},
+	{ "cyrerev" , CmdCyrillicChar , 0xFD},
+	{ "cyryu" , CmdCyrillicChar , 0xFE},
+	{ "cyrya" , CmdCyrillicChar , 0xFF},
+	{ "", NULL, 0 }
+};
+
 /********************************************************************/
 /* commands for begin-end environments */
 /* only strings used in the form \begin{text} or \end{text} */
@@ -425,6 +566,7 @@ static CommandArray params[] = {
 	{"figure*", CmdFigure, FIGURE_1},
 	{"picture", CmdPicture, 0},
 	{"minipage", CmdMinipage, 0},
+	{"music", CmdMusic, 0},
 
 	{"quote", CmdQuote, QUOTE},
 	{"quotation", CmdQuote, QUOTATION},
@@ -433,11 +575,12 @@ static CommandArray params[] = {
 	{"itemize", CmdList, ITEMIZE},
 	{"description", CmdList, DESCRIPTION},
 	{"verbatim", CmdVerbatim, VERBATIM_1},
+	{"comment", CmdVerbatim, VERBATIM_4},
 	{"verse", CmdVerse, 0},
 	{"tabular", CmdTabular, TABULAR},
-	{"tabular*", CmdTabular, TABULAR_1},
-	{"longtable", CmdTabular, TABULAR},
-	{"longtable*", CmdTabular, TABULAR_1},
+	{"tabular*", CmdTabular, TABULAR_STAR},
+	{"longtable", CmdTabular, TABULAR_LONG},
+	{"longtable*", CmdTabular, TABULAR_LONG_STAR},
 	{"array", CmdArray, 1},
 
 	{"displaymath", CmdEquation, EQN_DISPLAYMATH},
@@ -494,6 +637,90 @@ static CommandArray hyperlatex[] = {
 	{"", NULL, 0}
 };				/* end of list */
 
+/********************************************************************
+purpose: commands for apacite package 
+********************************************************************/
+static CommandArray apaciteCommands[] = {
+	{ "BBOP",   CmdApaCite,  0}, /* Open parenthesis Default is "(" */
+	{ "BBAA",   CmdApaCite,  1}, /* Last ``and'' Default is "\&" */
+	{ "BBAB",   CmdApaCite,  2}, /* Last ``and'' Default is "and" */
+	{ "BBAY",   CmdApaCite,  3}, /* Punctuation  Default is ", " */
+	{ "BBC",    CmdApaCite,  4}, /* Punctuation  Default is "; " */
+	{ "BBN",    CmdApaCite,  5}, /* Punctuation Default is ", " */
+	{ "BBCP",   CmdApaCite,  6}, /* Closing parenthesis, Default is ")" */
+	{ "BBOQ",   CmdApaCite,  7}, /* Opening quote Default is the empty string */
+	{ "BBCQ",   CmdApaCite,  8}, /* Closing quote Default is the empty string */
+	{ "BCBT",   CmdApaCite,  9}, /* Comma Default is "," */
+	{ "BCBL",   CmdApaCite, 10}, /* Comma Default is "," */
+	{ "BOthers",CmdApaCite, 11}, /* Used for ``others'' Default is "et~al." */
+	{ "BIP",    CmdApaCite, 12}, /* ``In press'', Default is "in press" */
+	{ "BAnd",   CmdApaCite, 13}, /* Used as ``and'' Default is "and" */
+	{ "BED",    CmdApaCite, 14}, /* Editor Default is "Ed." */
+	{ "BEDS",   CmdApaCite, 15}, /* Editors Default is "Eds." */
+	{ "BTRANS", CmdApaCite, 16}, /* Translator. Default is "Trans." */
+	{ "BTRANSS",CmdApaCite, 17}, /* Translators. Default is "Trans." */
+	{ "BCHAIR", CmdApaCite, 18}, /* Chair Default is "Chair" */
+	{ "BCHAIRS",CmdApaCite, 19}, /* Chairs. Default is "Chairs" */
+	{ "BVOL",   CmdApaCite, 20}, /* Volume,  Default is "Vol." */
+	{ "BVOLS",  CmdApaCite, 21}, /* Volumes, Default is "Vols." */
+	{ "BNUM",   CmdApaCite, 22}, /* Number, Default is "No." */
+	{ "BNUMS",  CmdApaCite, 23}, /* Numbers, Default is "Nos." */
+	{ "BEd",    CmdApaCite, 24}, /* Edition, Default is "ed." */
+	{ "BPG",    CmdApaCite, 25}, /* Page, default is "p." */
+	{ "BPGS",   CmdApaCite, 26}, /* Pages, default is "pp." */
+	{ "BTR",    CmdApaCite, 27}, /* technical report Default is "Tech.\ Rep." */
+	{ "BPhD",   CmdApaCite, 28}, /* Default is "Doctoral dissertation" */
+	{ "BUPhD",  CmdApaCite, 29}, /* Unpublished PhD Default is "Unpublished doctoral dissertation" */
+	{ "BMTh",   CmdApaCite, 30}, /* MS thesis Default is "Master's thesis" */
+	{ "BUMTh",  CmdApaCite, 31}, /* unpublished MS Default is "Unpublished master's thesis" */
+	{ "BOWP",   CmdApaCite, 32}, /* default is "Original work published " */
+	{ "BREPR",  CmdApaCite, 33}, /* default is "Reprinted from " */
+	{ "BCnt",   CmdApaCite, 34}, /* convert number to letter */
+	{ "BCntIP", CmdApaCite, 34}, /* convert number to letter */
+	{ "BBA",    CmdApaCite, 35}, /* "&" in paren, "and" otherwise */
+	{ "AX",     CmdApaCite, 36}, /* index name */
+	{ "Bem",    CmdEmphasize, F_EMPHASIZE_2},	
+	{ "BCAY",   CmdBCAY, 0},
+	{"fullcite", CmdCite, CITE_FULL},
+	{"shortcite", CmdCite, CITE_SHORT},
+	{"citeNP", CmdCite, CITE_CITE_NP},
+	{"fullciteNP", CmdCite, CITE_FULL_NP},
+	{"shortciteNP", CmdCite, CITE_SHORT_NP},
+	{"citeA", CmdCite, CITE_CITE_A},
+	{"fullciteA", CmdCite, CITE_FULL_A},
+	{"shortciteA", CmdCite, CITE_SHORT_A},
+	{"citeauthor", CmdCite, CITE_CITE_AUTHOR},
+	{"fullciteauthor", CmdCite, CITE_FULL_AUTHOR},
+	{"shortciteauthor", CmdCite, CITE_SHORT_AUTHOR},
+	{"citeyear", CmdCite, CITE_YEAR},
+	{"citeyearNP", CmdCite, CITE_YEAR_NP},
+	{"", NULL, 0}
+};
+
+/********************************************************************
+purpose: commands for apacite package 
+********************************************************************/
+static CommandArray natbibCommands[] = {
+	{"citet", CmdCite, CITE_T},
+	{"citet*", CmdCite, CITE_T_STAR},
+	{"citep", CmdCite, CITE_P},
+	{"citep*", CmdCite, CITE_P_STAR},
+	{"citealt", CmdCite, CITE_ALT},
+	{"citealp", CmdCite, CITE_ALP},
+	{"citealt*", CmdCite, CITE_ALT_STAR},
+	{"citealp*", CmdCite, CITE_ALP_STAR},
+	{"citetext", CmdCite, CITE_TEXT},
+	{"citeauthor", CmdCite, CITE_AUTHOR},
+	{"citeauthor*", CmdCite, CITE_AUTHOR_STAR},
+	{"citeyear", CmdCite, CITE_YEAR},
+	{"citeyearpar", CmdCite, CITE_YEAR_P},
+	{"Citet", CmdCite, CITE_T},
+	{"Citep", CmdCite, CITE_P},
+	{"Citealt", CmdCite, CITE_ALT},
+	{"Citealp", CmdCite, CITE_ALP},
+	{"Citeauthor", CmdCite, CITE_AUTHOR},
+	{"", NULL, 0}
+};
 
 bool 
 CallCommandFunc(char *cCommand)
@@ -542,7 +769,7 @@ globals: command-functions have side effects or recursive calls
 }
 
 
-bool 
+void 
 CallParamFunc(char *cCommand, int AddParam)
 /****************************************************************************
 purpose: Try to call the environment-function for the commandname
@@ -559,18 +786,17 @@ globals: command-functions have side effects or recursive calls
 		if (strcmp(params[i].cpCommand, cCommand) == 0) {
 			assert(params[i].func != NULL);
 			(*params[i].func) ((params[i].param) | AddParam);
-			return TRUE;	/* command function found */
+			return;	/* command function found */
 		}
 		++i;
 	}
 
 	/* unknown environment must be ignored */
 	if (AddParam == ON) {
-		sprintf(unknown_environment, "\\%s%s%s", "end{", cCommand, "}");
+		snprintf(unknown_environment, 100, "\\%s%s%s", "end{", cCommand, "}");
 		Ignore_Environment(cCommand);
 		diagnostics(WARNING, "Environment <%s> ignored.  Not defined in commands.c", cCommand);
 	}
-	return FALSE;
 }
 
 int 
@@ -594,6 +820,10 @@ globals: changes Environment - array of active environments
 {
 	char           *diag = "";
 
+	g_par_indent_array[iEnvCount] = getLength("parindent");
+	g_left_indent_array[iEnvCount] = g_left_margin_indent;
+	g_right_indent_array[iEnvCount] = g_right_margin_indent;
+	
 	switch (code) {
 	case PREAMBLE:
 		Environments[iEnvCount] = PreambleCommands;
@@ -611,10 +841,6 @@ globals: changes Environment - array of active environments
 		Environments[iEnvCount] = EnumerateCommands;
 		diag = "enumerate";
 		break;
-	case TABBING:
-		Environments[iEnvCount] = TabbingCommands;
-		diag = "tabbing";
-		break;
 	case LETTER:
 		Environments[iEnvCount] = LetterCommands;
 		diag = "letter";
@@ -631,6 +857,14 @@ globals: changes Environment - array of active environments
 		Environments[iEnvCount] = FrenchModeCommands;
 		diag = "french";
 		break;
+	case RUSSIAN_MODE:
+		Environments[iEnvCount] = RussianModeCommands;
+		diag = "french";
+ 		break;
+	case CZECH_MODE:
+		Environments[iEnvCount] = CzechModeCommands;
+		diag = "french";
+		break;
 	case FIGURE_ENV:
 		Environments[iEnvCount] = FigureCommands;
 		diag = "figure";
@@ -643,6 +877,18 @@ globals: changes Environment - array of active environments
 		Environments[iEnvCount] = hyperlatex;
 		diag = "hyperlatex";
 		break;
+	case APACITE_MODE:
+		Environments[iEnvCount] = apaciteCommands;
+		diag = "apacite";
+		break;
+	case NATBIB_MODE:
+		Environments[iEnvCount] = natbibCommands;
+		diag = "natbib";
+		break;
+	case GENERIC_ENV:
+		Environments[iEnvCount] = commands;
+		diag = "Generic Environment";
+		break;
 
 	default:
 		diagnostics(ERROR,"assertion failed at function PushEnvironment");
@@ -664,6 +910,10 @@ globals: changes Environment - array of active environments
 	--iEnvCount;
 	Environments[iEnvCount] = NULL;
 
+	setLength("parindent",g_par_indent_array[iEnvCount]);
+	g_left_margin_indent=g_left_indent_array[iEnvCount];
+	g_right_margin_indent=g_right_indent_array[iEnvCount];
+
 	/*
 	 * overlapping environments are not allowed !!! example:
 	 * \begin{verse}\begin{enumerate}\end{verse}\end{enumerate} ==>
