@@ -1,5 +1,23 @@
-/* $Id: convert.c,v 1.28 2002/04/04 03:11:24 prahl Exp $ 
-	
+/* convert.c - high level routines for LaTeX to RTF conversion
+
+Copyright (C) 2002 The Free Software Foundation
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 2
+of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+This file is available from http://sourceforge.net/projects/latex2rtf/
+
 TeX has six modes:
 	
 	MODE_VERTICAL              Building the main vertical list, from which the 
@@ -49,24 +67,30 @@ correctly, as well as vertical and horizontal space.
 #include "counters.h"
 #include "preamble.h"
 
-static bool     TranslateCommand();	/* converts commands */
+static void     TranslateCommand();	/* converts commands */
 
 static int    ret = 0;
 static int g_TeX_mode = MODE_VERTICAL;
 
-char TexModeName[6][25] = {"vertical", "internal vertical", "horizontal", 
-                        "restricted horizontal", "math", "displaymath"};
+char TexModeName[7][25] = {"bad", "internal vertical", "horizontal", 
+                        "restricted horizontal", "math", "displaymath","vertical"};
 
 void SetTexMode(int mode)
 {
-	if (g_TeX_mode != mode) 
-		diagnostics(4, "TeX mode now %s", TexModeName[mode]);
+	if (abs(mode) != g_TeX_mode) 
+		diagnostics(4, "TeX mode changing from [%s] -> [%s]", TexModeName[g_TeX_mode], TexModeName[abs(mode)]);
 		
-	if (g_TeX_mode == MODE_VERTICAL && mode == MODE_HORIZONTAL) {
-		g_TeX_mode = mode;  /* prevent recursion */
-		CmdStartParagraph(0);
+	if (mode < 0) {         /* hack to allow CmdStartParagraph to set mode directly */
+		g_TeX_mode = -mode;
+		return;
 	}
+
+	if (g_TeX_mode == MODE_VERTICAL && mode == MODE_HORIZONTAL) 
+		CmdStartParagraph(ANY_PAR);
 	
+	if (g_TeX_mode == MODE_HORIZONTAL && mode == MODE_VERTICAL)
+		CmdEndParagraph(0);
+
 	g_TeX_mode = mode;
 }
 
@@ -81,15 +105,22 @@ ConvertString(char *string)
      purpose : converts string in TeX-format to Rtf-format
  ******************************************************************************/
 {
-	char            temp[51];
+	char            temp[256];
 	
 	if (string==NULL) return;
 	
-	strncpy(temp,string,50);
-
-	if (PushSource(NULL, string)) {
-		diagnostics(4, "Entering Convert() from StringConvert() <%s>",temp);
-
+	if (strlen(string)<250)
+		strncpy(temp,string,250);
+	else {
+		strncpy(temp,string,125);
+		strncpy(temp+125,"\n...\n", 5);
+		strncpy(temp+130,string+strlen(string)-125,125);
+		temp[255] = '\0';
+	}
+	
+	if (PushSource(NULL, string) == 0) {
+		diagnostics(4, "Entering Convert() from StringConvert()\n******\n%s\n*****",temp);
+	
 		while (StillSource())
 			Convert();
 			
@@ -112,10 +143,10 @@ ConvertAllttString(char *s)
 	if (s==NULL) return;
 	diagnostics(4, "Entering Convert() from StringAllttConvert()");
 
-	if (PushSource(NULL, s)) {
+	if(PushSource(NULL, s)==0) {
 
 		while (StillSource()) {
-
+	
 			cThis = getRawTexChar();   /* it is a verbatim like environment */
 			switch (cThis) {
 			
@@ -142,7 +173,6 @@ ConvertAllttString(char *s)
 		}
 		PopSource();
 	}
-
 	diagnostics(4, "Exiting Convert() from StringAllttConvert()");
 }
 
@@ -156,8 +186,8 @@ globals: fTex, fRtf and all global flags for convert (see above)
 	char            cThis = '\n';
 	char            cLast = '\0';
 	char            cNext;
-	int				mode, count;
-
+	int				mode, count,pending_new_paragraph;
+	
 	diagnostics(3, "Entering Convert ret = %d", ret);
 	RecursionLevel++;
 	PushLevels();
@@ -171,6 +201,8 @@ globals: fTex, fRtf and all global flags for convert (see above)
 
 		mode = GetTexMode();
 		
+		pending_new_paragraph--;
+		
 		switch (cThis) {
 
 		case '\\':
@@ -190,6 +222,8 @@ globals: fTex, fRtf and all global flags for convert (see above)
 			
 			
 		case '{':
+			if (mode==MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
+				
 			CleanStack();
 			PushBrace();
 			fprintRTF("{");
@@ -233,6 +267,7 @@ globals: fTex, fRtf and all global flags for convert (see above)
 				cNext = getNonSpace(); 
 				
 				if (cNext == '\n') {			/* new paragraph ... skip all ' ' and '\n' */
+					pending_new_paragraph=2;
 					CmdEndParagraph(0);
 					cNext = getNonBlank();  	
 					ungetTexChar(cNext);
@@ -290,9 +325,8 @@ globals: fTex, fRtf and all global flags for convert (see above)
 			}
 			
 			if (g_processing_tabular) {	/* in tabular */
-				fprintRTF(" \\cell \\pard \\intbl ");
-				fprintRTF("\\q%c ", colFmt[actCol]);
 				actCol++;
+				fprintRTF("\\cell\\pard\\intbl\\q%c ", colFmt[actCol]);
 				break;
 			}
 			fprintRTF("&");
@@ -329,6 +363,13 @@ globals: fTex, fRtf and all global flags for convert (see above)
 			}
 			break;
 						
+		case '|':
+			if (mode == MODE_MATH || mode == MODE_DISPLAYMATH) 
+				fprintRTF("|");
+			else 
+				fprintRTF("\\emdash ");
+			break;
+
 		case '\'':
 			if (mode == MODE_MATH || mode == MODE_DISPLAYMATH)
 					fprintRTF("'");	
@@ -361,11 +402,21 @@ globals: fTex, fRtf and all global flags for convert (see above)
 
 		case '<':
 			if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
-			if (GetTexMode() == MODE_HORIZONTAL && FrenchMode ){
+			if (GetTexMode() == MODE_HORIZONTAL){
 				cNext = getTexChar();
-				if (cNext == '<')
-					fprintRTF("\\'ab");
-				else {
+				if (cNext == '<') {
+					if (FrenchMode) {			/* not quite right */
+						skipSpaces();
+						cNext = getTexChar();
+						if (cNext == '~') 
+							skipSpaces();
+						else
+							ungetTexChar(cNext);
+						fprintRTF("\\'ab\\~");
+					
+					} else
+						fprintRTF("\\'ab");
+				} else {
 					ungetTexChar(cNext);
 					fprintRTF("<");
 				}
@@ -376,7 +427,7 @@ globals: fTex, fRtf and all global flags for convert (see above)
 
 		case '>':
 			if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
-			if (GetTexMode() == MODE_HORIZONTAL && FrenchMode ){
+			if (GetTexMode() == MODE_HORIZONTAL){
 				cNext = getTexChar();
 				if (cNext == '>')
 					fprintRTF("\\'bb");
@@ -417,7 +468,10 @@ globals: fTex, fRtf and all global flags for convert (see above)
 				fprintRTF(":");	
 			else {
 				SetTexMode(MODE_HORIZONTAL);
-				fprintRTF(": ");
+				if (FrenchMode) 
+					fprintRTF("\\~:");
+  				else
+					fprintRTF(":");
 			}
 			break;	
 
@@ -430,7 +484,7 @@ globals: fTex, fRtf and all global flags for convert (see above)
 	
 				/* try to simulate double spaces after sentences */
 				cNext = getTexChar();
-				if (cNext == ' ' && (isalpha((int)cLast) && !isupper((int) cLast)))
+				if (0 && cNext == ' ' && (isalpha((int)cLast) && !isupper((int) cLast)))
 					fprintRTF(" ");
 				ungetTexChar(cNext);
 			}
@@ -452,14 +506,14 @@ globals: fTex, fRtf and all global flags for convert (see above)
 			break;
 
 		case '(':
-			if (g_processing_fields)
+			if (g_processing_fields&&g_escape_parent)
 				fprintRTF("\\\\(");
 			else
 				fprintRTF("(");
 			break;
 			
 		case ')':
-			if (g_processing_fields)
+			if (g_processing_fields&&g_escape_parent)
 				fprintRTF("\\\\)");
 			else
 				fprintRTF(")");
@@ -469,7 +523,10 @@ globals: fTex, fRtf and all global flags for convert (see above)
 			if (g_field_separator == ';' && g_processing_fields)
 				fprintRTF("\\\\;");
 			else
-				fprintRTF(";");
+				if (FrenchMode) 
+					fprintRTF("\\~;");
+  				else
+					fprintRTF(";");
 		break;
 
 		case ',':
@@ -501,19 +558,19 @@ globals: fTex, fRtf and all global flags for convert (see above)
 	diagnostics(5, "Exiting Convert via exhaustion ret = %d", ret);
 }
 
-bool 
+void 
 TranslateCommand()
 /****************************************************************************
 purpose: The function is called on a backslash in input file and
 	 tries to call the command-function for the following command.
-returns: sucess or not
+returns: success or not
 globals: fTex, fRtf, command-functions have side effects or recursive calls;
          global flags for convert
  ****************************************************************************/
 {
 	char            cCommand[MAXCOMMANDLEN];
 	int             i,mode;
-	int            cThis;
+	int             cThis;
 
 
 	cThis = getTexChar();
@@ -525,162 +582,163 @@ globals: fTex, fRtf, command-functions have side effects or recursive calls;
 	case '}':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
 		fprintRTF("\\}");
-		return TRUE;
+		return;
 	case '{':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
 		fprintRTF("\\{");
-		return TRUE;
+		return;
 	case '#':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
 		fprintRTF("#");
-		return TRUE;
+		return;
 	case '$':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
 		fprintRTF("$");
-		return TRUE;
+		return;
 	case '&':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
 		fprintRTF("&");
-		return TRUE;
+		return;
 	case '%':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
 		fprintRTF("%%");
-		return TRUE;
+		return;
 	case '_':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
 		fprintRTF("_");
-		return TRUE;
+		return;
 		
 	case '\\':		/* \\[1mm] or \\*[1mm] possible */
 		CmdSlashSlash(0);
-		return TRUE;
+		return;
 		
 	case ' ':
+	case '\n':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
 		fprintRTF(" ");	/* ordinary interword space */
 		skipSpaces();
-		return TRUE;
+		return;
 
 /* \= \> \< \+ \- \' \` all have different meanings in a tabbing environment */
 
 	case '-':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
-		if (tabbing_on){
+		if (g_processing_tabbing){
 			(void) PopBrace();
 			PushBrace();
 		} else
 			fprintRTF("\\-");
-		return TRUE;
+		return;
 
 	case '+':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
-		if (tabbing_on){
+		if (g_processing_tabbing){
 			(void) PopBrace();
 			PushBrace();
 		}
-		return TRUE;
+		return;
 		
 	case '<':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
-		if (tabbing_on){
+		if (g_processing_tabbing){
 			(void) PopBrace();
 			PushBrace();
 		}
-		return TRUE;
+		return;
 
 	case '>':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
-		if (tabbing_on){
+		if (g_processing_tabbing){
 			(void) PopBrace();
 			CmdTabjump();
 			PushBrace();
 		} else 
 			CmdSpace(0.50);  /* medium space */
-		return TRUE;
+		return;
 		
 	case '`':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
-		if (tabbing_on){
+		if (g_processing_tabbing){
 			(void) PopBrace();
 			PushBrace();
 		} else
 			CmdLApostrophChar(0);
-		return TRUE;
+		return;
 		
 	case '\'':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
-		if (tabbing_on){
+		if (g_processing_tabbing){
 			(void) PopBrace();
 			PushBrace();
-			return TRUE;
+			return;
 		} else
 			CmdRApostrophChar(0);	/* char ' =?= \' */
-		return TRUE;
+		return;
 
 	case '=':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
-		if (tabbing_on){
+		if (g_processing_tabbing){
 			(void) PopBrace();
 			CmdTabset();
 			PushBrace();
 		}
 		else
 			CmdMacronChar(0);
-		return TRUE;
+		return;
 		
 	case '~':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
 		CmdTildeChar(0);
-		return TRUE;
+		return;
 	case '^':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
 		CmdHatChar(0);
-		return TRUE;
+		return;
 	case '.':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
 		CmdDotChar(0);
-		return TRUE;
+		return;
 	case '\"':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
 		CmdUmlauteChar(0);
-		return TRUE;
+		return;
 	case '(':
 		CmdEquation(EQN_RND_OPEN | ON);
 		/*PushBrace();*/
-		return TRUE;
+		return;
 	case '[':
 		CmdEquation(EQN_BRACKET_OPEN | ON);
 		/*PushBrace();*/
-		return TRUE;
+		return;
 	case ')':
 		CmdEquation(EQN_RND_CLOSE | OFF);
 		/*ret = RecursionLevel - PopBrace();*/
-		return TRUE;
+		return;
 	case ']':
 		CmdEquation(EQN_BRACKET_CLOSE | OFF);
 		/*ret = RecursionLevel - PopBrace();*/
-		return TRUE;
+		return;
 	case '/':
 		CmdIgnore(0);		/* italic correction */
-		return TRUE;
+		return;
 	case '!':
 		CmdIgnore(0);		/* \! negative thin space */
-		return TRUE;
+		return;
 	case ',':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
 		CmdSpace(0.33);	/* \, produces a small space */
-		return TRUE;
+		return;
 	case ';':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
 		CmdSpace(0.75);	/* \; produces a thick space */
-		return TRUE;
+		return;
 	case '@':
 		CmdIgnore(0);	/* \@ produces an "end of sentence" space */
-		return TRUE;
+		return;
 	case '3':
 		if (mode == MODE_VERTICAL) SetTexMode(MODE_HORIZONTAL);
 		fprintRTF("{\\'df}");	/* german symbol '' */
-		return TRUE;
+		return;
 	}
 
 
@@ -713,7 +771,7 @@ globals: fTex, fRtf, command-functions have side effects or recursive calls;
 	diagnostics(5, "TranslateCommand() <%s>", cCommand);
 
 	if (i == 0)
-		return FALSE;
+		return;
 		
 	if (strcmp(cCommand,"begin")==0){
 		fprintRTF("{");
@@ -725,13 +783,12 @@ globals: fTex, fRtf, command-functions have side effects or recursive calls;
 			ret = RecursionLevel - PopBrace();
 			fprintRTF("}");
 		}
-		return TRUE;
+		return;
 	}
-	if (TryDirectConvert(cCommand, fRtf))
-		return TRUE;
-	if (TryVariableIgnore(cCommand))
-		return TRUE;
+	
+	if (TryDirectConvert(cCommand)) return;
+	
+	if (TryVariableIgnore(cCommand)) return;
 		
 	diagnostics(WARNING, "Command \\%s not found - ignored", cCommand);
-	return FALSE;
 }
