@@ -1,10 +1,25 @@
-/* $Id: preamble.c,v 1.30 2002/03/31 17:13:11 prahl Exp $
+/* preamble.c - Handles LaTeX commands that should only occur in the preamble.
 
-purpose : Handles LaTeX commands that should only occur in the preamble.
-          These are gathered together because the entire preamble must be
-		  parsed before the RTF header can be written.
-		  
-		  When \begin{document} is encountered, then the RTF header is created.
+Copyright (C) 2001-2002 The Free Software Foundation
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 2
+of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+This file is available from http://sourceforge.net/projects/latex2rtf/
+ 
+Authors:
+    2001-2002 Scott Prahl
 */
 
 #include <stdlib.h>
@@ -22,8 +37,9 @@ purpose : Handles LaTeX commands that should only occur in the preamble.
 #include "ignore.h"
 #include "commands.h"
 #include "counters.h"
-
-extern bool   pagestyledefined;
+#include "xref.h"
+#include "direct.h"
+#include "style.h"
 
 static bool   g_preambleTwoside  = FALSE;
 static bool   g_preambleTwocolumn= FALSE;
@@ -58,13 +74,30 @@ setPackageBabel(char * option)
 			ReadLanguage("german");
 	}
 	
-	if (strcmp(option, "french") == 0)
+	if (strcmp(option, "french") == 0 ||
+	    strcmp(option, "frenchb") == 0)
 	{
 		FrenchMode = TRUE;
 		PushEnvironment(FRENCH_MODE);
 		ReadLanguage("french");
 	}
 		
+	if (strcmp(option, "russian") == 0)
+	{
+		RussianMode = TRUE;
+		PushEnvironment(RUSSIAN_MODE);
+		ReadLanguage("russian");
+	}
+
+	if (strcmp(option, "czech") == 0)
+	{
+		CzechMode = TRUE;
+		PushEnvironment(CZECH_MODE);
+		ReadLanguage("czech");
+		g_fcharset_number=238;  			/* East European in RTF Specification */
+		strcpy(g_charset_encoding_name, "raw");
+	}
+		
 }
 
 void
@@ -73,26 +106,70 @@ setPackageInputenc(char * option)
 	g_preambleEncoding = strdup_noblanks(option);
 
 	if (strcmp(option, "ansinew") == 0)
-		strcpy(g_encoding, "cp1252");
+		strcpy(g_charset_encoding_name, "cp1252");
 		
 	else if (strcmp(option, "applemac") == 0 ||
-	         strcmp(option, "cp437") == 0 ||
-	         strcmp(option, "cp437de") == 0 ||
-	         strcmp(option, "cp850") == 0 ||
-	         strcmp(option, "cp852") == 0 ||
-	         strcmp(option, "cp865") == 0 ||
 	         strcmp(option, "decmulti") == 0 ||
-	         strcmp(option, "cp1250") == 0 ||
-	         strcmp(option, "cp1252") == 0 ||
-	         strcmp(option, "latin1") == 0 ||
-	         strcmp(option, "latin2") == 0 ||
-	         strcmp(option, "latin3") == 0 ||
-	         strcmp(option, "latin4") == 0 ||
-	         strcmp(option, "latin5") == 0 ||
-	         strcmp(option, "latin9") == 0 ||
-	         strcmp(option, "next") == 0 ) 
-		strcpy(g_encoding, option);
-	else
+	         strcmp(option, "latin1") == 0   ||
+	         strcmp(option, "latin2") == 0   ||
+	         strcmp(option, "latin3") == 0   ||
+	         strcmp(option, "latin4") == 0   ||
+	         strcmp(option, "latin5") == 0   ||
+	         strcmp(option, "latin9") == 0   ||
+	         strcmp(option, "next") == 0     ||
+	         strcmp(option, "cp437") == 0    ||
+	         strcmp(option, "cp437de") == 0  ||
+	         strcmp(option, "cp850") == 0    ||
+	         strcmp(option, "cp852") == 0    ||
+	         strcmp(option, "cp855") == 0    ||
+	         strcmp(option, "cp865") == 0    ||
+	         strcmp(option, "cp866") == 0    ||
+	         strcmp(option, "cp1250") == 0   ||
+	         strcmp(option, "cp1251") == 0   ||
+	         strcmp(option, "cp1252") == 0   ||
+	         strcmp(option, "437") == 0    ||
+	         strcmp(option, "437de") == 0  ||
+	         strcmp(option, "850") == 0    ||
+	         strcmp(option, "852") == 0    ||
+	         strcmp(option, "855") == 0    ||
+	         strcmp(option, "865") == 0    ||
+	         strcmp(option, "866") == 0    ||
+	         strcmp(option, "1250") == 0   ||
+	         strcmp(option, "1251") == 0   ||
+	         strcmp(option, "1252") == 0   ||
+	         strcmp(option, "maccyr") == 0   ||
+	         strcmp(option, "macukr") == 0   ||
+	         strcmp(option, "koi8-r") == 0   ||
+	         strcmp(option, "koi8-u") == 0) {
+	         
+		strcpy(g_charset_encoding_name, option);
+		g_fcharset_number = 0;				/* ANSI in RTF Specification */
+
+	} else if (strcmp(option, "raw") == 0) {
+		strcpy(g_charset_encoding_name, "raw");
+		g_fcharset_number = 255;            /* OEM in RTF Specification */
+
+	} else if (strcmp(option, "raw437") == 0 ) { 
+		g_fcharset_number=254;  			/* IBM PC in RTF Specification */
+		strcpy(g_charset_encoding_name, "raw");
+
+	} else if (strcmp(option, "raw852") == 0) { 
+		g_fcharset_number=255;  			/* Microsoft bug ... */
+		strcpy(g_charset_encoding_name, "raw");
+
+	} else if (strcmp(option, "raw1250") == 0 ) { 
+		g_fcharset_number=238;  			/* East European in RTF Specification */
+		strcpy(g_charset_encoding_name, "raw");
+
+	} else if (strcmp(option, "raw1251") == 0 ) { 
+		g_fcharset_number=204;  			/* Cyrillic in RTF Specification */
+		strcpy(g_charset_encoding_name, "raw");
+
+	} else if (strcmp(option, "raw1253") == 0 ) { 
+		g_fcharset_number=161;  			/* Greek in RTF Specification */
+		strcpy(g_charset_encoding_name, "raw");
+
+	} else
 		diagnostics(WARNING,"\n Input Encoding <%s> not supported", option);
 }
 
@@ -289,15 +366,15 @@ setPointSize(char * option)
 		setLength("parindent",   15*20);
 		setLength("parskip",      0*20);
 
-	}else if (strcmp(option, "11pt") == 0){
+	} else if (strcmp(option, "11pt") == 0){
 		InitializeDocumentFont(-1, 22, -1, -1);
 		setLength("baselineskip",14*20);
 		setLength("parindent",   17*20);
 		setLength("parskip",      0*20);
 
-	}else {
+	} else {
 		InitializeDocumentFont(-1, 24, -1, -1);
-		setLength("baselineskip",14.5*20);
+		setLength("baselineskip",(int) 14.5*20);
 		setLength("parindent",   18*20);
 		setLength("parskip",      0*20);
 	}
@@ -316,7 +393,7 @@ setDocumentOptions(char *optionlist)
 	while (option) {
 
 /*		while (*option == ' ') option++;  skip leading blanks */
-		diagnostics(4, "                    option   <%s>", option);
+		diagnostics(3, " (setDocumentOptions) option <%s>", option);
 		if      (strcmp(option, "10pt"       ) == 0 ||
 			     strcmp(option, "11pt"       ) == 0 || 
 				 strcmp(option, "12pt"       ) == 0) 
@@ -333,6 +410,9 @@ setDocumentOptions(char *optionlist)
 		else if (strcmp(option, "german")  == 0 ||
 			     strcmp(option, "spanish") == 0 || 
 			     strcmp(option, "english") == 0 || 
+			     strcmp(option, "russian") == 0 || 
+			     strcmp(option, "czech"  ) == 0 || 
+			     strcmp(option, "frenchb"  ) == 0 || 
 				 strcmp(option, "french")  == 0) 
 			setPackageBabel(option);
 		else if (strcmp(option, "twoside") == 0) 
@@ -343,11 +423,37 @@ setDocumentOptions(char *optionlist)
 			g_preambleTitlepage = TRUE;
 		else if (strcmp(option, "isolatin1") == 0)
 			setPackageInputenc("latin1");
-		else if (strcmp(option, "hyperlatex") == 0) {
+		else if (strcmp(option, "hyperlatex") == 0)
 			PushEnvironment(HYPERLATEX); 
+		else if (strcmp(option, "apalike") == 0)
+			g_document_bibstyle = BIBSTYLE_APALIKE;
+		else if (strcmp(option, "apanat1b") == 0)
+			PushEnvironment(APACITE_MODE);
+		else if (strcmp(option, "natbib") == 0){
+			PushEnvironment(NATBIB_MODE);
+			g_document_bibstyle = BIBSTYLE_NATBIB;
+		} else if (strcmp(option, "authordate") == 0 ||
+			     strcmp(option, "authordate1") == 0 ||
+			     strcmp(option, "authordate2") == 0 ||
+			     strcmp(option, "authordate3") == 0 ||
+			     strcmp(option, "authordate4") == 0 ||
+			     strcmp(option, "authordate1-4") == 0 ){
+	/*		PushEnvironment(AUTHORDATE_MODE);*/
+			g_document_bibstyle = BIBSTYLE_AUTHORDATE;
+		} else if (strcmp(option, "apacite")==0   ||
+			     strcmp(option, "apacitex") == 0 ) {
+			PushEnvironment(APACITE_MODE);
+			g_document_bibstyle = BIBSTYLE_APACITE;
 		} else if (strcmp(option, "fancyhdr") == 0) {
 			diagnostics(WARNING, "Only partial support for %s", option);
-		} else if (!TryVariableIgnore(option)) {
+		} else if (strcmp(option, "textcomp")==0 ||
+				   strcmp(option, "fontenc")==0) {
+			/* do nothing ... but don't complain */
+		}
+		else if (strcmp(option, "color") == 0) {
+			diagnostics(WARNING, "Color support limited to eight basic colors");
+		}
+		else if (!TryVariableIgnore(option)) {
 			diagnostics(WARNING, "Unknown style option %s ignored", option);
 		}
 		option = strtok(NULL, ",");
@@ -430,11 +536,13 @@ CmdUsepackage(int code)
 	else if (strcmp(package, "isolatin1") == 0)
 		setPackageInputenc("latin1");
 
-	else if (strcmp(package, "babel") == 0 && options)
-		setPackageBabel(options);
-		
-	else if (strcmp(package, "german")  == 0 ||
+	else if (strcmp(package, "babel") == 0) {
+		if (options) setPackageBabel(options);	
+			
+	} else if (strcmp(package, "german" )  == 0 ||
 		     strcmp(package, "ngerman")  == 0 ||
+		     strcmp(package, "czech"  )  == 0 ||
+		     strcmp(package, "frenchb")  == 0 ||
 		     strcmp(package, "french")  == 0) 
 		setPackageBabel(package);
 
@@ -447,8 +555,12 @@ CmdUsepackage(int code)
 	         strstr(package, "newcen")        ||
 	         strstr(package, "helvet") )
 		setPackageFont(package);
-		
-	else
+
+	else if (strcmp(package, "natbib") == 0) {
+		if (strstr(options, "")) set_longnamesfirst();
+		PushEnvironment(NATBIB_MODE);
+		g_document_bibstyle = BIBSTYLE_NATBIB;
+	} else
 		setDocumentOptions(package);
 	
 	if (options)
@@ -495,9 +607,9 @@ CmdMakeTitle(int code)
 	char            date_begin[10];
 
 	PushTrackLineNumber(FALSE);
-	sprintf(title_begin, "%s%2d", "\\fs", (30 * CurrentFontSize()) / 20);
-	sprintf(author_begin, "%s%2d", "\\fs", (24 * CurrentFontSize()) / 20);
-	sprintf(date_begin, "%s%2d", "\\fs", (24 * CurrentFontSize()) / 20);
+	snprintf(title_begin, 10, "%s%2d", "\\fs", (30 * CurrentFontSize()) / 20);
+	snprintf(author_begin, 10, "%s%2d", "\\fs", (24 * CurrentFontSize()) / 20);
+	snprintf(date_begin, 10, "%s%2d", "\\fs", (24 * CurrentFontSize()) / 20);
 
 	alignment = CENTERED;
 	fprintRTF("\n\\par\\pard\\qc {%s ", title_begin);
@@ -515,8 +627,9 @@ CmdMakeTitle(int code)
 		ConvertString(g_preambleDate);
 	fprintRTF("}");
 	
-	fprintRTF("\n\\par\n\\par\\pard\\q%c ", alignment);
+	CmdEndParagraph(0);
 	alignment = JUSTIFIED;
+
 	if (g_preambleTitlepage)
 		fprintRTF("\\page ");
 
@@ -573,7 +686,6 @@ CmdPagestyle( /* @unused@ */ int code)
 
  globals : headings  set to TRUE if the pagenumber is to go into the header
            pagenumbering set to TRUE if pagenumbering is to occur- default
-	   pagestyledefined, flag, set to true
 
 Produces latex-like headers and footers.
 Needs to be terminated for:
@@ -583,7 +695,6 @@ Needs to be terminated for:
 {
 	static char    *style = "";
 
-	pagestyledefined = TRUE;
 	style = getBraceParam();
 	if (strcmp(style, "empty") == 0) {
 		if (pagenumbering) {
@@ -683,27 +794,38 @@ WriteFontHeader(void)
 /****************************************************************************
  *   purpose: writes fontnumbers and styles for headers into Rtf-File
  
- \fcharset0:    ANSI coding
+ \fcharset0:    ANSI coding (codepage 1252)
  \fcharset1:    MAC coding
- \fcharset2:    PC coding (implies CodePage 437)
- \fcharset3:    PCA coding (implies CodePage 850)
+ \fcharset2:    PC coding   (codepage 437)
+ \fcharset3:    PCA coding  (codepage 850)
+ \fcharset204:  Cyrillic    (codepage 1251)
+ \fcharset238:  E. European (codepage 852, 1250)
  ****************************************************************************/
 {
-	int                  num = 3;
+	int                  i;
 	ConfigEntryT       **config_handle;
+	char                *font_type, *font_name;
+	int					 charset;
 
-	fprintRTF("{\\fonttbl\n");
+	fprintRTF("{\\fonttbl");
 
 	config_handle = CfgStartIterate(FONT_A);
+	i=3;
 	while ((config_handle = CfgNext(FONT_A, config_handle)) != NULL) {
-		if (strstr((*config_handle)->RtfCommand, "Symbol"))
+	
+		font_type = (char *)(*config_handle)->TexCommand;
+		font_name = (char *)(*config_handle)->RtfCommand;
+		charset = g_fcharset_number;
+		
+		if (strncmp(font_name, "Symbol", 6)==0)
+			charset = 2;
+			
+		if (strncmp(font_type, "Cyrillic", 8)==0)	
+			charset = 204;
+					
+		fprintRTF("{\\f%d\\fnil\\fcharset%d %s;}\n",i, charset, font_name);
 
-			fprintRTF(" {\\f%d\\fnil\\fcharset2 %s;}\n",num,(*config_handle)->RtfCommand);
-
-		else
-			fprintRTF(" {\\f%d\\fnil\\fcharset0 %s;}\n",num,(*config_handle)->RtfCommand);
-
-		++num;
+		i++;
 	}
 
 	fprintRTF("}\n");
@@ -729,18 +851,20 @@ WriteStyleHeader(void)
          \par}
  ****************************************************************************/
 {
-	int DefFont = DefaultFontFamily();
-	fprintRTF("{\\stylesheet{\\fs%d\\lang1024\\snext0 Normal;}\n", CurrentFontSize());
-	fprintRTF("{%s\\f%d%s \\sbasedon0\\snext0 heading 1;}\n", HEADER11, DefFont, HEADER12);
-	fprintRTF("{%s\\f%d%s \\sbasedon0\\snext0 heading 2;}\n", HEADER21, DefFont, HEADER22);
-	fprintRTF("{%s\\f%d%s \\sbasedon0\\snext0 heading 3;}\n", HEADER31, DefFont, HEADER32);
-	fprintRTF("{%s\\f%d%s \\sbasedon0\\snext0 heading 4;}\n", HEADER41, DefFont, HEADER42);
-	fprintRTF("%s\n", HEADER03);
+	ConfigEntryT **style;
+	const char *rtf;
 	
-	fprintRTF("%s\n", HEADER13);
-	fprintRTF("%s\n", HEADER23);
-	fprintRTF("%s\n", HEADER33);
-	fprintRTF("%s\n", HEADER43);
+	fprintRTF("{\\stylesheet\n{\\s0\\fs%d\\snext0 Normal;}\n", CurrentFontSize());
+	
+	style = CfgStartIterate(STYLE_A);
+	while ((style = CfgNext(STYLE_A, style)) != NULL) {
+		rtf = (*style)->RtfCommand;
+		diagnostics(4,"style <%s>=<%s>", (*style)->TexCommand,rtf);
+		fprintRTF("{");
+		InsertBasicStyle(rtf, TRUE);
+		fprintRTF(";}\n");
+	}
+	fprintRTF("}\n");
 }
 
 static void
@@ -850,7 +974,13 @@ void CmdHeadFoot(int code)
  adapted from code by Taupin in ltx2rtf
  ******************************************************************************/
 {
-  char *HeaderText = getBraceParam();
+  char *HeaderText = NULL;
+  char *AlternText = NULL;
+  
+  AlternText = getBracketParam();
+  if (AlternText) free(AlternText);
+  
+  HeaderText = getBraceParam();
   
   diagnostics(4,"CmdHeadFoot code=%d <%s>",code, HeaderText);
   switch(code)
@@ -880,10 +1010,23 @@ WriteColorTable(void)
      <colordef>          \red ? & \green ? & \blue ? ';'
  ***************************************************************************/
 {
-	fprintRTF("{\\colortbl");
-	fprintRTF("\\red255\\green0\\blue0;");
-	fprintRTF("\\red0\\green255\\blue0;");
-	fprintRTF("\\red0\\green0\\blue255;");
+	fprintRTF("{\\colortbl;\n");
+	fprintRTF("\\red0\\green0\\blue0;\n");      /* black */
+	fprintRTF("\\red0\\green0\\blue255;\n");    /* blue */
+	fprintRTF("\\red0\\green255\\blue255;\n");  /* cyan */
+	fprintRTF("\\red0\\green255\\blue0;\n");    /* green */
+	fprintRTF("\\red255\\green0\\blue255;\n");  /* magenta */
+	fprintRTF("\\red255\\green0\\blue0;\n");    /* red */
+	fprintRTF("\\red255\\green255\\blue0;\n");  /* yellow */
+	fprintRTF("\\red255\\green255\\blue255;\n");/* white */
+	fprintRTF("\\red0\\green0\\blue128;\n");    /* dark blue */
+	fprintRTF("\\red0\\green128\\blue128;\n");  /* dark cyan */
+	fprintRTF("\\red0\\green128\\blue0;\n");    /* dark green */
+	fprintRTF("\\red128\\green0\\blue128;\n");  /* dark magenta */
+	fprintRTF("\\red128\\green0\\blue0;\n");    /* dark red */
+	fprintRTF("\\red128\\green128\\blue0;\n");  /* dark yellow */
+	fprintRTF("\\red128\\green128\\blue128;\n");/* dark gray */
+	fprintRTF("\\red192\\green192\\blue192;\n");/* light gray */
 	fprintRTF("}\n");
 }
 
@@ -897,7 +1040,7 @@ WriteInfo(void)
   \keywords       Selected keywords for the document
   \comment        Comments; text is ignored
   \version<N>     The version number of the document
-  \doccomm        Comments displayed in Words Summary Info dialog
+  \doccomm        Comments displayed in Word's Summary Info dialog
   
 {\info {\title This is a page} {\author \'ca}}
  ***************************************************************************/
@@ -913,11 +1056,12 @@ purpose: writes header info for the RTF file
  ****************************************************************************/
 {
 	int family = DefaultFontFamily();
-	int size   = DefaultFontSize();
+/*	int size   = DefaultFontSize(); */
 
 	diagnostics(4, "Writing header for RTF file");
 
-	fprintRTF("{\\rtf1\\ansi\\fs%d\\deff%d\\deflang1024\n", size, family);
+/*	fprintRTF("{\\rtf1\\ansi\\fs%d\\deff%d\\deflang1024\n", size, family); */
+	fprintRTF("{\\rtf1\\ansi\\deff%d\\deflang1024\n", family);
 	WriteFontHeader();
 	WriteColorTable();
 	WriteStyleHeader();
